package com.fr.plugin.cafe_expand.function;

import com.fr.general.ComparatorUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.script.AbstractFunction;
import com.fr.stable.StringUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;

@FunctionRecorder(localeKey="FS_PLUGIN_CAFE_EXPAND")
public class DoExpandFile extends AbstractFunction {
    public static Map<String,Map<Integer, int[]>> colExpandMap=new HashMap();
    public static Map<String,Map<Integer, int[]>> rowExpandMap=new HashMap();

    public Object run(Object[] arg) {

        String result="模板文件不存在";
        String path=this.getClass().getResource("/").getPath();
        path=path.substring(0, path.indexOf("WEB-INF") + 7) + "/reportlets";

        String filepath=arg[0].toString();
        String position=arg[1].toString();
        String colexpand=arg[2].toString();
        String rowexpand=arg[3].toString();
        path=path+"/"+filepath;

        File f= new File(path);
        if(f.exists()) {

            Map<Integer, int[]> colMap = getMap(colexpand);
            Map<Integer, int[]> rowMap = getMap(rowexpand);
            colExpandMap.put(path, colMap);
            rowExpandMap.put(path, rowMap);

            String newPath = getNewPath(path);
            try {
                Element ele = start(path, position);
                FileOutputStream out = new FileOutputStream(newPath, true);
                out.write(ele.asXML().getBytes("utf-8"));
                out.flush();
                out.close();

                result = newPath.substring(newPath.indexOf("reportlets") + 11);
                rowExpandMap.remove(path);
                colExpandMap.remove(path);
            } catch (Exception e) {
                result = e.getMessage();
            }
        }
        return result;
    }

    private static Element start(String filePath,String position)
    {
        Element root = UnitTools.getRootEle(filePath);
        int sheetNum=0;

        if(filePath.contains(".cpt"))
        {
            //position格式  sheet:1,blockName:block1
            String sheetArr[]=position.split(",");
            sheetNum=Integer.parseInt(sheetArr[0].split(":")[1]);

            int i=1;
            List<Element> sheetReports = root.elements();
            for (Element reportEle : sheetReports) {
                String className = reportEle.attributeValue("class");
                if ("Report".equals(reportEle.getName()) && className.contains("WorkSheet")) {
                    if(i==sheetNum) {//只分析当前编辑的sheet页
                        if (className.contains("PolyWorkSheet"))//聚合报表
                        {
                            String editBlock=sheetArr[1].split(":")[1];

                            List<Element> polyReports = reportEle.elements();
                            for (Element polyEle : polyReports) {
                                if (polyEle.getName().equals("Block")) {
                                    String blockName = "";
                                    try {
                                        blockName = polyEle.element("PECBAttr").attributeValue("blockName");
                                    } catch (Exception e) {
                                        blockName = polyEle.element("PCBAttr").attributeValue("blockName");
                                    }

                                    if(ComparatorUtils.equals(editBlock,blockName)) {//只分析当前编辑的block
                                        String bolckClass = polyEle.attributeValue("class");
                                        if (!bolckClass.contains("PolyChartBlock"))
                                            doExpandCafe(polyEle,filePath);
                                        break;
                                    }
                                }
                            }
                        } else//普通报表
                            doExpandCafe(reportEle,filePath);

                        break;
                    }
                    i++;
                }
            }
        }
        else
        {
            Element Layout=root.element("Layout");
            if(Layout!=null)////解析报表主体
            {
                String arr[]=position.split(";");//frm:report0
                if(ComparatorUtils.equals(arr[1],"frm"))
                    anaFrm(Layout,arr[1],filePath);
            }
        }

        //填报属性替换单元格
        Element webEle=root.element("ReportWebAttr");
        if(webEle!=null)
            analysisEle(webEle,filePath,sheetNum);

        return root;
    }





    /**
     * 解析 决策报表
     * */
    private static void anaFrm(Element rootEle,String reportName,String filePath)
    {
        Element centerEle=rootEle;
        List<Element> widgetList=centerEle.elements();
        for(Element widget:widgetList)
        {
            if("Widget".equals(widget.getName()))
            {
                String widgetKind="";
                Element inner=widget.element("InnerWidget");
                if(inner==null)
                    inner=widget;
                String widgetName=inner.element("WidgetName").attributeValue("name");
                String className=inner.attributeValue("class");
                if(className.contains("WAbsolute"))
                    widgetKind="绝对画布块";
                else if(className.contains("WCardMainBorderLayout"))
                {
                    widgetKind="temp";
                    inner=inner.element("Center");
                }
                else if(className.contains("WTabFitLayout"))
                    widgetKind="Tab标签";
                else if(className.contains("WTitleLayout"))
                    widgetKind="temp";

                if(!"".equals(widgetKind))
                    anaFrm(inner,reportName,filePath);

                else {
                    if (className.contains("ElementCaseEditor") && ComparatorUtils.equals(widgetName,reportName)) {
                        Element report=inner.element("FormElementCase");
                        doExpandCafe(report,filePath);
                        break;
                    }
                }
            }
        }
    }


    /**
     * 报表块 模拟行列扩展
     * 修改单元格 c、r、cs、rs的值
     * */
    private static void doExpandCafe(Element reportRoot,String filePath)
    {
        List elepar = reportRoot.getParent().content();
        int index = elepar.indexOf(reportRoot);

        //变换单元格
        reportRoot=analysisEle(reportRoot,filePath,-1);

        //执行扩展行列
        List<Element> cellList=reportRoot.element("CellElementList").elements();
        for(Element cell:cellList) {
            int col = Integer.parseInt(cell.attributeValue("c"));
            int row = Integer.parseInt(cell.attributeValue("r"));
            String cs = cell.attributeValue("cs");//横向合并单元格
            String rs = cell.attributeValue("rs");//纵向合并单元格
            int csnum=0;
            if(cs!=null)
                csnum=Integer.parseInt(cs);
            int rsnum=0;
            if(rs!=null)
                rsnum=Integer.parseInt(rs);

            Map<Integer,int[]> colMap=colExpandMap.get(filePath);
            ListIterator<Map.Entry<Integer,int[]>> i=new ArrayList(colMap.entrySet()).listIterator(colMap.size());
            while(i.hasPrevious()) {
                Map.Entry<Integer,int[]> entry=i.previous();
                int colexpand[]=entry.getValue();
                if (colexpand[2] != 0) {
                    if (colexpand[1] == 0) {
                        //插入列
                        if (colexpand[0] <= col) {
                            col = col + colexpand[2];
                            cell.attribute("c").setValue(col + "");
                        } else if (colexpand[0] < (col + csnum)) {
                            csnum = csnum + colexpand[2];
                            cell.attribute("cs").setValue(csnum + "");
                        }
                    } else {
                        //删除列
                        int delete_c_Num = colexpand[0] + colexpand[2];
                        if (colexpand[0] < col) {
                            if (delete_c_Num <= col)//不破坏当前单元格，位置往上移动
                            {
                                col=col - colexpand[2];
                                cell.attribute("c").setValue(col + "");
                            }
                            //破坏部分单元格，扩展数要变动，同时位置往上移动
                            else if (delete_c_Num >= col && delete_c_Num < (col + csnum)) {
                                int dim = delete_c_Num - col;
                                col=colexpand[0];
                                csnum=csnum - dim;
                                cell.attribute("c").setValue(col + "");
                                cell.attribute("cs").setValue(csnum + "");
                            } else if (delete_c_Num >= (col + csnum))//破坏整个单元格 删除
                            {
                                cell.getParent().remove(cell);
                                continue;
                            }
                        } else if (colexpand[0] == col)//正好从单元格所在行列开始删除
                        {
                            if (delete_c_Num >= (col + csnum))//破坏整个单元格 删除
                            {
                                cell.getParent().remove(cell);
                                continue;
                            } else//破坏部分单元格，扩展数要变动，单元格位置不变
                            {
                                csnum=csnum - colexpand[2];
                                cell.attribute("cs").setValue(csnum + "");
                            }
                        }
                        //从单元格中间开始删除,单元格位置不变，只有扩展数变动
                        else if (colexpand[0] > col && colexpand[0] < (col + csnum)) {
                            int num = csnum - colexpand[2];
                            if (delete_c_Num > (col + csnum))
                                num = num + delete_c_Num - (col + csnum);
                            if (num < 0)
                                num = 0;
                            csnum=num;
                            cell.attribute("cs").setValue(csnum + "");
                        }
                        //从单元格下面删除，不影响当前单元格
                        else if (colexpand[0] >= (col + csnum)) {
                            //跳过
                        }
                    }
                }
            }

            Map<Integer,int[]> rowMap=rowExpandMap.get(filePath);
            ListIterator<Map.Entry<Integer,int[]>> ii=new ArrayList(rowMap.entrySet()).listIterator(rowMap.size());
            while(ii.hasPrevious()) {
                Map.Entry<Integer,int[]> entry=ii.previous();
                int rowexpand[]=entry.getValue();
                if (rowexpand[2] != 0) {
                    if (rowexpand[1] == 0) {
                        //插入行
                        if (rowexpand[0] <= row) {
                            row = row + rowexpand[2];
                            cell.attribute("r").setValue(row + "");
                        } else if (rowexpand[0] < (row + rsnum)) {
                            rsnum = rsnum + rowexpand[2];
                            cell.attribute("rs").setValue(rsnum + "");
                        }
                    } else {
                        //删除行
                        int delete_r_Num = rowexpand[0] + rowexpand[2];
                        if (rowexpand[0] < row) {
                            if (delete_r_Num <= row)//不破坏当前单元格，位置往上移动
                            {
                                row=row - rowexpand[2];
                                cell.attribute("r").setValue(row + "");
                            }
                                //破坏部分单元格，扩展数要变动，同时位置往上移动
                            else if (delete_r_Num >= row && delete_r_Num < (row + rsnum)) {
                                int dim = delete_r_Num - row;
                                row=rowexpand[0];
                                rsnum=rsnum - dim;
                                cell.attribute("r").setValue(row + "");
                                cell.attribute("rs").setValue(rsnum + "");
                            } else if (delete_r_Num >= (row + rsnum))//破坏整个单元格 删除
                            {
                                cell.getParent().remove(cell);
                                continue;
                            }
                        } else if (rowexpand[0] == row)//正好从单元格所在行列开始删除
                        {
                            if (delete_r_Num >= (row + rsnum))//破坏整个单元格 删除
                            {
                                cell.getParent().remove(cell);
                                continue;
                            } else//破坏部分单元格，扩展数要变动，单元格位置不变
                            {
                                rsnum=rsnum - rowexpand[2];
                                cell.attribute("rs").setValue(rsnum  + "");
                            }
                        }
                        //从单元格中间开始删除,单元格位置不变，只有扩展数变动
                        else if (rowexpand[0] > row && rowexpand[0] < (row + rsnum)) {
                            int num = rsnum - rowexpand[2];
                            if (delete_r_Num > (row + rsnum))
                                num = num + delete_r_Num - (row + rsnum);
                            if (num < 0)
                                num = 1;
                            rsnum=num;
                            cell.attribute("rs").setValue(num + "");
                        }
                        //从单元格下面删除，不影响当前单元格
                        else if (rowexpand[0] >= (row + rsnum)) {
                            //跳过
                        }
                    }
                }
            }
        }

        try {
            //替换后的，转换成新的ele，替换原来的report
            Document new_ele = DocumentHelper.parseText(reportRoot.asXML());
            elepar.set(index,new_ele.getRootElement());

        } catch (DocumentException e) {
            e.printStackTrace();
        }
    }


    /**
     * 逐行解析ele，变更需要扩展的内容
     * */
    private static Element analysisEle(Element ele, String filePath, int sheetNum)
    {
        String arr[]=ele.asXML().split("\n");
        StringBuilder sb=new StringBuilder();
        Map<String,Boolean> judgeMap=new HashMap();

        //逐行解析字符串，将需要变更的单元格存入map
        Map<String, String> map=new LinkedHashMap();
        for(String lineTxt:arr)
        {
            anaStr(lineTxt , filePath , map);
            anaJSstr(lineTxt , filePath , map);
        }

        //map排序
        Map<String, String> newmap=new LinkedHashMap();
        UnitTools.sortExpandCafe(map,newmap);


        //逐行解析字符串，替换单元格
        for(String lineTxt:arr)
        {
            for (Map.Entry<String, String> entry : newmap.entrySet()) {
                String cafeStr=entry.getKey();

                //判断是否包含了需要变更的单元格
                int index=lineTxt.indexOf(cafeStr);

                if(index>-1)
                {
                    boolean judge=true;
                    if(sheetNum > -1) {
                        if (judgeMap.containsKey(cafeStr))
                            judge = judgeMap.get(cafeStr);
                        else {
                            judge = UnitTools.judgeSheet(cafeStr, sheetNum);
                            judgeMap.put(cafeStr, judge);
                        }
                    }
                    if(judge)
                        lineTxt= UnitTools.replaceCafe(lineTxt,cafeStr,entry.getValue());
                }
            }

            sb.append(lineTxt);sb.append("\n");
        }

        Element result=null;
        try {
            //替换后的，转换成新的ele，替换原来的report
            Document new_ele = DocumentHelper.parseText(sb.toString());

            List elepar = ele.getParent().content();
            elepar.set(elepar.indexOf(ele),new_ele.getRootElement());
            result=new_ele.getRootElement();

        } catch (DocumentException e) {
            e.printStackTrace();
        }
        return result;
    }


    /**
     * 解析字符串，获取单元格，判断单元格是否需要扩展
     * 需要扩展的元素存入map
     * */
    private static void anaStr(String str,String filePath,Map<String, String> m)
    {
        List<String> cafeList=new ArrayList();
        UnitTools.getCellCafe(str,cafeList);
        for(int i=0;i<cafeList.size();i++)
        {
            String cafe=cafeList.get(i);
            String newCafe=getNewCafe(cafe,filePath);
            if(!ComparatorUtils.equals(cafe,newCafe))
            {
                //将变化的单元格存入map
                m.put(cafe,newCafe);
                m.put(UnitTools.getcafeStr(cafe), UnitTools.getcafeStr(newCafe));
            }
        }
    }


    /**
     * 获取 扩展后的 单元格
     * */
    private static String getNewCafe(String cafe,String filePath)
    {
        int colRow[] = UnitTools.getColRow(cafe);

        int newCol=getNewCol(colRow[0],filePath);
        int newRow=getNewRow(colRow[1],filePath);

        return UnitTools.getCafe("" + newCol, newRow + "");
    }


    /**
     * 获取扩展后的列
     * */
    private static int getNewCol(int col,String filePath)
    {
        Map<Integer,int[]> m=colExpandMap.get(filePath);

        ListIterator<Map.Entry<Integer,int[]>> i=new ArrayList(m.entrySet()).listIterator(m.size());
        while(i.hasPrevious()) {
            Map.Entry<Integer,int[]> entry=i.previous();
            int c=entry.getKey();
            int arr[]=entry.getValue();
            if(col>c)//当前单元格列数大于插入列
            {
                if (arr[1] == 0)
                    col = col + arr[2];
                else if (arr[1] == 1)
                    col = col - arr[2];
            }
        }
        return  col;
    }


    /**
     * 获取扩展后的行
     * */
    private static int getNewRow(int row,String filePath)
    {
        Map<Integer,int[]> m=rowExpandMap.get(filePath);

        ListIterator<Map.Entry<Integer,int[]>> i=new ArrayList(m.entrySet()).listIterator(m.size());
        while(i.hasPrevious()) {
            Map.Entry<Integer,int[]> entry=i.previous();
            int r = entry.getKey();
            int arr[] = entry.getValue();

            if (row > r)//当前单元格行数大于插入行
            {
                if (arr[1] == 0)
                    row = row + arr[2];
                else if (arr[1] == 1)
                    row = row - arr[2];
            }
        }
        return row;
    }


    /**
     * 解析js中涉及的的单元格
     * 如setCellValue(a,b,c); getCellValue(a,b,c)
     * */
    private static void anaJSstr(String js,String filePath, Map<String, String> map)
    {
        try {
            int start1 = js.indexOf("e.getCellValue");
            if (start1 > 0)
                anaJSValue(js, start1, filePath,map);
            else {
                int start2 = js.indexOf("e.setCellValue");
                if (start2 > 0)
                    anaJSValue(js, start2, filePath, map);
            }
        }catch (Exception e){}
    }


    /**
     * 获取 变更后的js字符串，存入map
     * */
    private static void anaJSValue(String str,int start,String filePath ,Map<String, String> map)
    {
        String tempStr=str.substring(start+2);
        int end=tempStr.indexOf(")");
        String value=tempStr.substring(0,end);

        String tempValue=value.substring(value.indexOf("(")+1);
        String arr[]=tempValue.split(",");
        String newStr="";
        if(value.contains("get"))
        {
            if(arr.length==3) {
                int col = Integer.parseInt(arr[1].replaceAll(" ", ""));
                int row = Integer.parseInt(arr[2].replaceAll(" ", ""));

                col = getNewCol(col, filePath);
                row = getNewRow(row, filePath);
                newStr = arr[0] + "," + col + "," + row;
            }
            else if(arr.length==2)
            {
                int col = Integer.parseInt(arr[0].replaceAll(" ", ""));
                int row = Integer.parseInt(arr[1].replaceAll(" ", ""));

                col = getNewCol(col, filePath);
                row = getNewRow(row, filePath);
                newStr = col + "," + row;
            }
        }
        else if(value.contains("set"))
        {
            int col=Integer.parseInt(arr[0].replaceAll(" ",""));
            int row=Integer.parseInt(arr[1].replaceAll(" ",""));

            col=getNewCol(col,filePath);
            row=getNewRow(row,filePath);
            newStr=col+","+row+","+arr[2];
        }

        if(!StringUtils.equals(tempValue,newStr))
            //将变化的单元格存入map
            map.put("(" + tempValue + ")", "(" + newStr + ")");


        int totalEnd=start+end+2;
        anaJSstr(str.substring(totalEnd),filePath,map);
    }



    /**
     * 获取新文件路径
     * */
    private static String getNewPath(String path)
    {
        SimpleDateFormat sdf = new SimpleDateFormat("yyMMddHHmmss");
        String nowtime = sdf.format(new Date());
        String newPath=path.substring(0,path.length()-4)+"_expand_"+nowtime+path.substring(path.length()-4);

        return newPath;
    }

    /**
     * 获取扩展的行列 1:1:1;
     * 第一个，代表从哪开始
     * 第二个，代表插入还是删除（1删除，0插入）
     * 第三个，代表扩展的数量
     * */
    private static Map<Integer,int[]> getMap(String str)
    {
        Map<Integer,int[]> map=new HashMap();
        try {
            if (!StringUtils.isEmpty(str)) {
                String arr[] = str.split(",");
                for (String s : arr) {
                    String a[] = s.split(":");
                    map.put(Integer.parseInt(a[0]) - 1,getExpandInfo(s));
                }
            }
        }catch (Exception e){};
        return map;
    }

    private static int[] getExpandInfo(String str)
    {
        int arr[]={0,0,0};
        if(!StringUtils.isEmpty(str))
        {
            try{
                String a[]=str.split(":");
                for(int i=0;i<3;i++)
                    arr[i]=Integer.parseInt(a[i]);
            }catch (Exception e){}
        }
        return arr;
    }
}
